"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mathViewAndEditPlugin = exports.MathBlockNodeView = exports.MathInlineNodeView = exports.MathNodeView = void 0;
const tslib_1 = require("tslib");
const katex_1 = tslib_1.__importDefault(require("katex"));
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_keymap_1 = require("prosemirror-keymap");
const prosemirror_view_1 = require("prosemirror-view");
const selection_1 = require("../../../utils/selection");
const commands_1 = require("./commands");
const const_1 = require("./const");
require("katex/dist/katex.min.css");
class MathNodeView {
    constructor(node) {
        this.node = node;
        this.texContent = this.getTexContent();
        const elems = this.createDOM();
        this.dom = elems.dom;
        this.contentDOM = elems.contentDOM;
        this.mathViewDOM = elems.mathViewDOM;
        this.renderKatex();
    }
    update(node) {
        if (this.node.type !== node.type)
            return false;
        this.node = node;
        const newTexContent = this.getTexContent();
        if (this.texContent !== newTexContent) {
            this.texContent = newTexContent;
            this.renderKatex();
        }
        return true;
    }
    renderKatex() {
        try {
            katex_1.default.render(this.texContent, this.mathViewDOM, {
                throwOnError: false,
                displayMode: this.isDisplayMode(),
            });
        }
        catch (err) {
            const errorElem = document.createElement('span');
            errorElem.classList.add('math-error');
            errorElem.innerText = `(error) ${this.texContent}`;
            errorElem.title = String(err);
            this.mathViewDOM.appendChild(errorElem);
        }
    }
    getTexContent() {
        return this.node.textContent;
    }
}
exports.MathNodeView = MathNodeView;
class MathInlineNodeView extends MathNodeView {
    isDisplayMode() {
        return false;
    }
    createDOM() {
        const dom = document.createElement('span');
        dom.classList.add('math-container', 'math-inline-container');
        const mathViewDOM = document.createElement('span');
        mathViewDOM.classList.add('math-view', 'math-inline-view');
        mathViewDOM.contentEditable = 'false';
        const mathInlineDOM = this.createMathInlineDOM();
        dom.appendChild(mathViewDOM);
        dom.appendChild(mathInlineDOM.container);
        return { dom, contentDOM: mathInlineDOM.content, mathViewDOM };
    }
    // same as math-inline spec toDOM()
    createMathInlineDOM() {
        const container = document.createElement('span');
        container.classList.add(const_1.CLASSNAMES.Inline.Container);
        const sharpBefore = document.createElement('span');
        sharpBefore.classList.add(const_1.CLASSNAMES.Inline.Sharp);
        sharpBefore.contentEditable = 'false';
        sharpBefore.textContent = '$';
        const sharpAfter = sharpBefore.cloneNode(true);
        const content = document.createElement('span');
        content.classList.add(const_1.CLASSNAMES.Inline.Content);
        container.replaceChildren(sharpBefore, content, sharpAfter);
        return { container, content };
    }
}
exports.MathInlineNodeView = MathInlineNodeView;
class MathBlockNodeView extends MathNodeView {
    isDisplayMode() {
        return true;
    }
    createDOM() {
        const dom = document.createElement('div');
        dom.classList.add('math-container', 'math-block-container');
        const mathViewDOM = document.createElement('div');
        mathViewDOM.classList.add('math-view', 'math-block-view');
        mathViewDOM.contentEditable = 'false';
        const contentDOM = document.createElement('div');
        contentDOM.classList.add('math-block');
        dom.appendChild(mathViewDOM);
        dom.appendChild(contentDOM);
        return { dom, contentDOM, mathViewDOM };
    }
}
exports.MathBlockNodeView = MathBlockNodeView;
const mathViewAndEditPlugin = () => new prosemirror_state_1.Plugin({
    props: {
        nodeViews: {
            [const_1.MathNode.Block]: (node) => new MathBlockNodeView(node),
            [const_1.MathNode.Inline]: (node) => new MathInlineNodeView(node),
        },
        handleKeyDown: (0, prosemirror_keymap_1.keydownHandler)({
            ArrowLeft: commands_1.moveCursorToEndOfMathInline,
        }),
        decorations: (state) => {
            const { selection } = state;
            if ((0, selection_1.isTextSelection)(selection)) {
                const decorations = [];
                state.doc.nodesBetween(selection.from, selection.to, (node, pos) => {
                    const typeName = node.type.name;
                    if (typeName === const_1.MathNode.Inline || typeName === const_1.MathNode.Block) {
                        decorations.push(prosemirror_view_1.Decoration.node(pos, pos + node.nodeSize, {
                            class: 'math-active',
                        }));
                    }
                });
                if (decorations.length) {
                    return prosemirror_view_1.DecorationSet.create(state.doc, decorations);
                }
            }
            return prosemirror_view_1.DecorationSet.empty;
        },
    },
});
exports.mathViewAndEditPlugin = mathViewAndEditPlugin;
