"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.goToNextRow = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const table_utils_1 = require("../../../../table-utils");
const utils_1 = require("../../../../table-utils/utils");
const selection_1 = require("../../../../utils/selection");
function goToNextRow(dir) {
    return (state, dispatch, view) => {
        const parentTable = (0, utils_1.findParentTable)(state.selection);
        const parentRow = (0, table_utils_1.findParentTableRow)(state.selection);
        const parentCell = (0, utils_1.findParentTableCell)(state.selection);
        if (!view || !(0, table_utils_1.atEndOfCell)(view, dir === 'up' ? -1 : 1)) {
            return false;
        }
        if (!parentTable || !parentRow || !parentCell) {
            return false;
        }
        const allRows = (0, table_utils_1.findChildTableRows)(parentTable.node);
        const rowIndex = allRows.findIndex((node) => node.node === parentRow.node);
        let cellIndex;
        for (let i = 0; i < parentRow.node.childCount; i++) {
            if (parentRow.node.child(i) === parentCell.node)
                cellIndex = i;
        }
        if (cellIndex === undefined) {
            return false;
        }
        const newRowIndex = rowIndex + (dir === 'up' ? -1 : 1);
        if (newRowIndex < 0 || newRowIndex >= allRows.length) {
            (0, selection_1.createFakeParagraphNear)(dir, parentTable)(state, dispatch);
            return true;
        }
        const newRow = allRows[newRowIndex];
        const childCells = (0, utils_1.findChildTableCells)(newRow.node);
        const cell = childCells[cellIndex];
        const from = parentTable.start + newRow.pos + cell.pos + 3;
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.setSelection(new prosemirror_state_1.TextSelection(state.doc.resolve(from))).scrollIntoView());
        return true;
    };
}
exports.goToNextRow = goToNextRow;
