import { chainCommands, exitCode } from 'prosemirror-commands';
import { logger } from '../../../logger';
import { isMac } from '../../../utils/platform';
import { nodeTypeFactory } from '../../../utils/schema';
const hardBreak = 'hard_break';
const softBreak = 'soft_break';
export const hbType = nodeTypeFactory(hardBreak);
export const sbType = nodeTypeFactory(softBreak);
export const Breaks = (builder, opts) => {
    var _a;
    let preferredBreak;
    if (builder.context.has('breaks')) {
        preferredBreak = builder.context.get('breaks') ? 'soft' : 'hard';
    }
    else {
        preferredBreak = (_a = opts.preferredBreak) !== null && _a !== void 0 ? _a : 'hard';
        logger.info("[Breaks extension]: Parameter 'breaks' is not defined in context; value from options is used");
    }
    builder.addNode(hardBreak, () => ({
        spec: {
            inline: true,
            group: 'inline',
            selectable: false,
            parseDOM: [{ tag: 'br' }],
            toDOM() {
                return ['br'];
            },
        },
        fromYfm: { tokenName: 'hardbreak', tokenSpec: { name: hardBreak, type: 'node' } },
        toYfm: (state, node, parent, index) => {
            for (let i = index + 1; i < parent.childCount; i++) {
                if (parent.child(i).type !== node.type) {
                    state.write('\\\n');
                    return;
                }
            }
        },
    }));
    // TODO: should we handle softbreak differently at different md.options.breaks setting?
    // we can safely convert softbreak into hardbreak,
    // but in this case non-edited markup will always be changed – a backspash will be added
    builder.addNode(softBreak, () => ({
        spec: {
            inline: true,
            group: 'inline',
            selectable: false,
            toDOM() {
                return ['br'];
            },
        },
        fromYfm: { tokenName: 'softbreak', tokenSpec: { name: softBreak, type: 'node' } },
        toYfm: (state, node, parent, index) => {
            for (let i = index + 1; i < parent.childCount; i++) {
                if (parent.child(i).type !== node.type) {
                    state.write('\n');
                    return;
                }
            }
        },
    }));
    builder.addKeymap(({ schema }) => {
        const cmd = addBr((preferredBreak === 'soft' ? sbType : hbType)(schema));
        const keys = {
            'Shift-Enter': cmd,
        };
        if (isMac()) {
            keys['Ctrl-Enter'] = cmd;
        }
        return keys;
    });
};
const addBr = (br) => chainCommands(exitCode, (state, dispatch) => {
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.replaceSelectionWith(br.create()).scrollIntoView());
    return true;
});
